==> chromium-74.0.3693.0.tar.xz: chromium-74.0.3693.0/media/mojo/services/mojo_renderer_service.h <==
// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_MOJO_SERVICES_MOJO_RENDERER_SERVICE_H_
#define MEDIA_MOJO_SERVICES_MOJO_RENDERER_SERVICE_H_

#include <stdint.h>
#include <memory>

#include "base/callback.h"
#include "base/compiler_specific.h"
#include "base/macros.h"
#include "base/memory/ref_counted.h"
#include "base/memory/weak_ptr.h"
#include "base/timer/timer.h"
#include "base/unguessable_token.h"
#include "media/base/buffering_state.h"
#include "media/base/media_resource.h"
#include "media/base/pipeline_status.h"
#include "media/base/renderer_client.h"
#include "media/mojo/interfaces/renderer.mojom.h"
#include "media/mojo/services/media_mojo_export.h"
#include "mojo/public/cpp/bindings/strong_binding.h"

namespace media {

class CdmContextRef;
class MediaResourceShim;
class MojoCdmServiceContext;
class Renderer;

// A mojom::Renderer implementation that use a media::Renderer to render
// media streams.
class MEDIA_MOJO_EXPORT MojoRendererService : public mojom::Renderer,
                                              public RendererClient {
 public:
  using InitiateSurfaceRequestCB = base::Callback<base::UnguessableToken()>;

  // Helper function to bind MojoRendererService with a StrongBinding,
  // which is safely accessible via the returned StrongBindingPtr.
  static mojo::StrongBindingPtr<mojom::Renderer> Create(
      MojoCdmServiceContext* mojo_cdm_service_context,
      std::unique_ptr<media::Renderer> renderer,
      const InitiateSurfaceRequestCB& initiate_surface_request_cb,
      mojo::InterfaceRequest<mojom::Renderer> request);

  // |mojo_cdm_service_context| can be used to find the CDM to support
  // encrypted media. If null, encrypted media is not supported.
  MojoRendererService(MojoCdmServiceContext* mojo_cdm_service_context,
                      std::unique_ptr<media::Renderer> renderer,
                      InitiateSurfaceRequestCB initiate_surface_request_cb);

  ~MojoRendererService() final;

  // mojom::Renderer implementation.
  void Initialize(
      mojom::RendererClientAssociatedPtrInfo client,
      base::Optional<std::vector<mojom::DemuxerStreamPtrInfo>> streams,
      const base::Optional<GURL>& media_url,
      const base::Optional<GURL>& site_for_cookies,
      InitializeCallback callback) final;
  void Flush(FlushCallback callback) final;
  void StartPlayingFrom(base::TimeDelta time_delta) final;
  void SetPlaybackRate(double playback_rate) final;
  void SetVolume(float volume) final;
  void SetCdm(int32_t cdm_id, SetCdmCallback callback) final;
  void InitiateScopedSurfaceRequest(
      InitiateScopedSurfaceRequestCallback callback) final;

  void set_bad_message_cb(base::Closure bad_message_cb) {
    bad_message_cb_ = bad_message_cb;
  }

 private:
  enum State {
    STATE_UNINITIALIZED,
    STATE_INITIALIZING,
    STATE_FLUSHING,
    STATE_PLAYING,
    STATE_ERROR
  };

  // RendererClient implementation.
  void OnError(PipelineStatus status) final;
  void OnEnded() final;
  void OnStatisticsUpdate(const PipelineStatistics& stats) final;
  void OnBufferingStateChange(BufferingState state) final;
  void OnWaiting(WaitingReason reason) final;
  void OnAudioConfigChange(const AudioDecoderConfig& config) final;
  void OnVideoConfigChange(const VideoDecoderConfig& config) final;
  void OnVideoNaturalSizeChange(const gfx::Size& size) final;
  void OnVideoOpacityChange(bool opaque) final;
  void OnDurationChange(base::TimeDelta duration) final;
  void OnRemotePlayStateChange(MediaStatus::State state) final;

  // Called when the MediaResourceShim is ready to go (has a config,
  // pipe handle, etc) and can be handed off to a renderer for use.
  void OnStreamReady(base::OnceCallback<void(bool)> callback);

  // Called when |audio_renderer_| initialization has completed.
  void OnRendererInitializeDone(base::OnceCallback<void(bool)> callback,
                                PipelineStatus status);

  // Periodically polls the media time from the renderer and notifies the client
  // if the media time has changed since the last update.
  // If |force| is true, the client is notified even if the time is unchanged.
  // If |range| is true, an interpolation time range is reported.
  void UpdateMediaTime(bool force);
  void CancelPeriodicMediaTimeUpdates();
  void SchedulePeriodicMediaTimeUpdates();

  // Callback executed once Flush() completes.
  void OnFlushCompleted(FlushCallback callback);

  // Callback executed once SetCdm() completes.
  void OnCdmAttached(base::OnceCallback<void(bool)> callback, bool success);

  MojoCdmServiceContext* const mojo_cdm_service_context_ = nullptr;

  State state_;
  double playback_rate_;

  std::unique_ptr<MediaResource> media_resource_;

  base::RepeatingTimer time_update_timer_;
  base::TimeDelta last_media_time_;

  mojom::RendererClientAssociatedPtr client_;

  // Holds the CdmContextRef to keep the CdmContext alive for the lifetime of
  // the |renderer_|.
  std::unique_ptr<CdmContextRef> cdm_context_ref_;

  // Note: Destroy |renderer_| first to avoid access violation into other
  // members, e.g. |media_resource_| and |cdm_|.
  // Must use "media::" because "Renderer" is ambiguous.
  std::unique_ptr<media::Renderer> renderer_;

  // Registers a new request in the ScopedSurfaceRequestManager.
  // Returns the token to be used to fulfill the request.
  InitiateSurfaceRequestCB initiate_surface_request_cb_;

  // Callback to be called when an invalid or unexpected message is received.
  // TODO(tguilbert): Revisit how to do InitiateScopedSurfaceRequest() so that
  // we can eliminate this callback. See http://crbug.com/669606
  base::Closure bad_message_cb_;

  base::WeakPtr<MojoRendererService> weak_this_;
  base::WeakPtrFactory<MojoRendererService> weak_factory_;

  DISALLOW_COPY_AND_ASSIGN(MojoRendererService);
};

}  // namespace media

#endif  // MEDIA_MOJO_SERVICES_MOJO_RENDERER_SERVICE_H_

==> chromium-75.0.3759.8.tar.xz: chromium-75.0.3759.8/media/mojo/services/mojo_renderer_service.h <==
// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_MOJO_SERVICES_MOJO_RENDERER_SERVICE_H_
#define MEDIA_MOJO_SERVICES_MOJO_RENDERER_SERVICE_H_

#include <stdint.h>
#include <memory>

#include "base/callback.h"
#include "base/compiler_specific.h"
#include "base/macros.h"
#include "base/memory/ref_counted.h"
#include "base/memory/weak_ptr.h"
#include "base/timer/timer.h"
#include "base/unguessable_token.h"
#include "media/base/buffering_state.h"
#include "media/base/media_resource.h"
#include "media/base/pipeline_status.h"
#include "media/base/renderer_client.h"
#include "media/mojo/interfaces/renderer.mojom.h"
#include "media/mojo/services/media_mojo_export.h"
#include "mojo/public/cpp/bindings/strong_binding.h"

namespace media {

class CdmContextRef;
class MediaResourceShim;
class MojoCdmServiceContext;
class Renderer;

// A mojom::Renderer implementation that use a media::Renderer to render
// media streams.
class MEDIA_MOJO_EXPORT MojoRendererService : public mojom::Renderer,
                                              public RendererClient {
 public:
  // Helper function to bind MojoRendererService with a StrongBinding,
  // which is safely accessible via the returned StrongBindingPtr.
  static mojo::StrongBindingPtr<mojom::Renderer> Create(
      MojoCdmServiceContext* mojo_cdm_service_context,
      std::unique_ptr<media::Renderer> renderer,
      mojo::InterfaceRequest<mojom::Renderer> request);

  // |mojo_cdm_service_context| can be used to find the CDM to support
  // encrypted media. If null, encrypted media is not supported.
  MojoRendererService(MojoCdmServiceContext* mojo_cdm_service_context,
                      std::unique_ptr<media::Renderer> renderer);

  ~MojoRendererService() final;

  // mojom::Renderer implementation.
  void Initialize(
      mojom::RendererClientAssociatedPtrInfo client,
      base::Optional<std::vector<mojom::DemuxerStreamPtrInfo>> streams,
      const base::Optional<GURL>& media_url,
      const base::Optional<GURL>& site_for_cookies,
      bool allow_credentials,
      InitializeCallback callback) final;
  void Flush(FlushCallback callback) final;
  void StartPlayingFrom(base::TimeDelta time_delta) final;
  void SetPlaybackRate(double playback_rate) final;
  void SetVolume(float volume) final;
  void SetCdm(int32_t cdm_id, SetCdmCallback callback) final;

  // TODO(tguilbert): Get rid of |bad_message_cb_|, now that it's no longer
  // needed.
  void set_bad_message_cb(base::Closure bad_message_cb) {
    bad_message_cb_ = bad_message_cb;
  }

 private:
  enum State {
    STATE_UNINITIALIZED,
    STATE_INITIALIZING,
    STATE_FLUSHING,
    STATE_PLAYING,
    STATE_ERROR
  };

  // RendererClient implementation.
  void OnError(PipelineStatus status) final;
  void OnEnded() final;
  void OnStatisticsUpdate(const PipelineStatistics& stats) final;
  void OnBufferingStateChange(BufferingState state) final;
  void OnWaiting(WaitingReason reason) final;
  void OnAudioConfigChange(const AudioDecoderConfig& config) final;
  void OnVideoConfigChange(const VideoDecoderConfig& config) final;
  void OnVideoNaturalSizeChange(const gfx::Size& size) final;
  void OnVideoOpacityChange(bool opaque) final;
  void OnRemotePlayStateChange(MediaStatus::State state) final;

  // Called when the MediaResourceShim is ready to go (has a config,
  // pipe handle, etc) and can be handed off to a renderer for use.
  void OnStreamReady(base::OnceCallback<void(bool)> callback);

  // Called when |audio_renderer_| initialization has completed.
  void OnRendererInitializeDone(base::OnceCallback<void(bool)> callback,
                                PipelineStatus status);

  // Periodically polls the media time from the renderer and notifies the client
  // if the media time has changed since the last update.
  // If |force| is true, the client is notified even if the time is unchanged.
  // If |range| is true, an interpolation time range is reported.
  void UpdateMediaTime(bool force);
  void CancelPeriodicMediaTimeUpdates();
  void SchedulePeriodicMediaTimeUpdates();

  // Callback executed once Flush() completes.
  void OnFlushCompleted(FlushCallback callback);

  // Callback executed once SetCdm() completes.
  void OnCdmAttached(base::OnceCallback<void(bool)> callback, bool success);

  MojoCdmServiceContext* const mojo_cdm_service_context_ = nullptr;

  State state_;
  double playback_rate_;

  std::unique_ptr<MediaResource> media_resource_;

  base::RepeatingTimer time_update_timer_;
  base::TimeDelta last_media_time_;

  mojom::RendererClientAssociatedPtr client_;

  // Holds the CdmContextRef to keep the CdmContext alive for the lifetime of
  // the |renderer_|.
  std::unique_ptr<CdmContextRef> cdm_context_ref_;

  // Note: Destroy |renderer_| first to avoid access violation into other
  // members, e.g. |media_resource_| and |cdm_|.
  // Must use "media::" because "Renderer" is ambiguous.
  std::unique_ptr<media::Renderer> renderer_;

  // Callback to be called when an invalid or unexpected message is received.
  // TODO(tguilbert): Revisit how to do InitiateScopedSurfaceRequest() so that
  // we can eliminate this callback. See http://crbug.com/669606
  base::Closure bad_message_cb_;

  base::WeakPtr<MojoRendererService> weak_this_;
  base::WeakPtrFactory<MojoRendererService> weak_factory_;

  DISALLOW_COPY_AND_ASSIGN(MojoRendererService);
};

}  // namespace media

#endif  // MEDIA_MOJO_SERVICES_MOJO_RENDERER_SERVICE_H_
